@@ -12,17 +12,16 @@ class LayerNorm(nn.Module):
     def __init__(self, dim, eps = 1e-5):
         super().__init__()
         self.eps = eps
-        self.g = nn.Parameter(torch.ones(dim))
+        self.gamma = nn.Parameter(torch.ones(dim))
+        self.register_buffer('beta', torch.zeros(dim))
 
     def forward(self, x):
-        var = torch.var(x, dim = -1, unbiased = False, keepdim = True)
-        mean = torch.mean(x, dim = -1, keepdim = True)
-        return (x - mean) / (var + self.eps).sqrt() * self.g
+        return F.layer_norm(x, x.shape[-1:], self.gamma, self.beta)
 
 # parallel with residual
 # discovered by Wang et al + EleutherAI from GPT-J fame
 
-class Parallel(nn.Module):
+class ParallelResidual(nn.Module):
     def __init__(self, *fns):
         super().__init__()
         self.fns = nn.ModuleList(fns)
@@ -50,10 +49,7 @@ def rotate_half(x):
     return torch.cat((-x2, x1), dim = -1)
 
 def apply_rotary_pos_emb(pos, t):
-    rot_dim = pos.shape[-1]
-    t, t_pass = t[..., :rot_dim], t[..., rot_dim:]
-    t = (t * pos.cos()) + (rotate_half(t) * pos.sin())
-    return torch.cat((t, t_pass), dim = -1)
+    return (t * pos.cos()) + (rotate_half(t) * pos.sin())
 
 # feedforward
 # classic Noam Shazeer paper, except here they use SwiGLU instead of the more popular GEGLU
@@ -87,7 +83,7 @@ class Attention(nn.Module):
         self.norm = LayerNorm(dim)
         self.heads = heads
         self.scale = dim_head ** -0.5
-        self.rotary_emb = RotaryEmbedding(min(32, dim_head))
+        self.rotary_emb = RotaryEmbedding(dim_head)
 
         self.to_q = nn.Linear(dim, inner_dim, bias = False)
         self.to_kv = nn.Linear(dim, dim_head * 2, bias = False)
@@ -136,6 +132,7 @@ class Attention(nn.Module):
 
         # attention
 
+        sim = sim - sim.amax(dim = -1, keepdim = True).detach()
         attn = sim.softmax(dim = -1)
 
         # aggregate values
@@ -149,42 +146,26 @@ class Attention(nn.Module):
 
 # transformer
 
-class PaLM(nn.Module):
-    def __init__(
-        self,
-        *,
-        dim,
-        num_tokens,
-        depth,
-        dim_head = 64,
-        heads = 8,
-        ff_mult = 4
-    ):
-        super().__init__()
-        self.token_emb = nn.Embedding(num_tokens, dim)
+def PaLM(
+    *,
+    dim,
+    num_tokens,
+    depth,
+    dim_head = 64,
+    heads = 8,
+    ff_mult = 4
+):
+    net = nn.Sequential(
+        nn.Embedding(num_tokens, dim),
+        *[ParallelResidual(
+            Attention(dim = dim, dim_head = dim_head, heads = heads),
+            FeedForward(dim = dim, mult = ff_mult),
+        ) for _ in range(depth)],
+        LayerNorm(dim),
+        nn.Linear(dim, num_tokens, bias = False)
+    )
 
-        self.layers = nn.Sequential(*[
-            Parallel(
-                Attention(dim = dim, dim_head = dim_head, heads = heads),
-                FeedForward(dim = dim, mult = ff_mult),
-            ) for _ in range(depth)
-        ])
+    net[-1].weight = net[0].weight  # they used embedding weight tied projection out to logits, not common, but works
 
-        self.norm = LayerNorm(dim)
-
-    def forward(self, x):
-        """
-        einstein notation
-        b - batch
-        n - sequence
-        d - feature dimension
-        c - logits dimension
-        """
-
-        x = self.token_emb(x)
-
-        x = self.layers(x)
-        x = self.norm(x)
-
-        logits = einsum('b n d, c d -> b n c', x, self.token_emb.weight)
-        return logits
+    nn.init.normal_(net[0].weight, std = 0.02)
+    return net
